// ignore_for_file: non_constant_identifier_names
part of 'amap_track.dart';

class _IOSDelegate extends NSObject with AMapTrackManagerDelegate {
  ValueChanged<QueryTerminalResponse> _onQueryTerminalResponse;
  ValueChanged<int> _onAddTerminal;
  ValueChanged<int> _onAddTrack;
  VoidCallback _onStartService;
  VoidCallback _onStopService;
  VoidCallback _onStartGather;
  VoidCallback _onStopGather;
  ValueChanged<TrackPoint> _onQueryLastPoint;
  ValueChanged<num> _onQueryDistance;
  ValueChanged<HistoryTrack> _onQueryHistoryTrack;
  ValueChanged<List<Track>> _onQueryTrack;

  ValueChanged<TrackError> _onRequestFailed;

  @override
  Future<void> onStartService(AMapTrackErrorCode errorCode) async {
    super.onStartService(errorCode);
    debugPrint('启动服务');
    if (errorCode == AMapTrackErrorCode.AMapTrackErrorOK ||
        errorCode == AMapTrackErrorCode.AMapTrackErrorCodeGatherStarted) {
      if (_onStartService != null) _onStartService();
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(errorCode.toValue(), ''));
    }
  }

  @override
  Future<void> onStopService(AMapTrackErrorCode errorCode) async {
    super.onStopService(errorCode);
    debugPrint('停止服务');
    if (errorCode == AMapTrackErrorCode.AMapTrackErrorOK ||
        errorCode == AMapTrackErrorCode.AMapTrackErrorCodeServiceStopped) {
      if (_onStopService != null) _onStopService();
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(errorCode.toValue(), ''));
    }
  }

  @override
  Future<void> onStartGatherAndPack(AMapTrackErrorCode errorCode) async {
    super.onStartGatherAndPack(errorCode);
    debugPrint('开始收集定位信息');
    if (errorCode == AMapTrackErrorCode.AMapTrackErrorOK ||
        errorCode == AMapTrackErrorCode.AMapTrackErrorCodeGatherStarted) {
      if (_onStartGather != null) _onStartGather();
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(errorCode.toValue(), ''));
    }
  }

  @override
  Future<void> onStopGatherAndPack(AMapTrackErrorCode errorCode) async {
    super.onStopGatherAndPack(errorCode);
    debugPrint('停止收集定位信息');
    if (errorCode == AMapTrackErrorCode.AMapTrackErrorOK ||
        errorCode == AMapTrackErrorCode.AMapTrackErrorCodeGatherStopped) {
      if (_onStopGather != null) _onStopGather();
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(errorCode.toValue(), ''));
    }
  }

  @override
  Future<void> onQueryTerminalDone_response(
    AMapTrackQueryTerminalRequest request,
    AMapTrackQueryTerminalResponse response,
  ) async {
    super.onQueryTerminalDone_response(request, response);
    if (await response.get_code() == AMapTrackErrorCode.AMapTrackErrorOK) {
      if (_onQueryTerminalResponse != null) {
        _onQueryTerminalResponse(await QueryTerminalResponse.ios(response));
      }
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(
        (await response.get_code()).toValue(),
        await response.get_info(),
      ));
    }
  }

  @override
  Future<void> onAddTerminalDone_response(
    AMapTrackAddTerminalRequest request,
    AMapTrackAddTerminalResponse response,
  ) async {
    super.onAddTerminalDone_response(request, response);
    if (await response.get_code() == AMapTrackErrorCode.AMapTrackErrorOK) {
      if (_onAddTerminal != null) {
        _onAddTerminal(int.parse(await response.get_terminalID()));
      }
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(
        (await response.get_code()).toValue(),
        await response.get_info(),
      ));
    }
  }

  @override
  Future<void> onAddTrackDone_response(
    AMapTrackAddTrackRequest request,
    AMapTrackAddTrackResponse response,
  ) async {
    super.onAddTrackDone_response(request, response);
    if (await response.get_code() == AMapTrackErrorCode.AMapTrackErrorOK) {
      if (_onAddTrack != null) {
        _onAddTrack(int.tryParse(await response.get_trackID()));
      }
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(
        (await response.get_code()).toValue(),
        await response.get_info(),
      ));
    }
  }

  @override
  Future<void> onQueryLastPointDone_response(
    AMapTrackQueryLastPointRequest request,
    AMapTrackQueryLastPointResponse response,
  ) async {
    super.onQueryLastPointDone_response(request, response);
    if (await response.get_code() == AMapTrackErrorCode.AMapTrackErrorOK) {
      if (_onQueryLastPoint != null) {
        _onQueryLastPoint(await TrackPoint.ios(await response.get_lastPoint()));
      }
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(
        (await response.get_code()).toValue(),
        await response.get_info(),
      ));
    }
  }

  @override
  Future<void> onQueryTrackDistanceDone_response(
    AMapTrackQueryTrackDistanceRequest request,
    AMapTrackQueryTrackDistanceResponse response,
  ) async {
    super.onQueryTrackDistanceDone_response(request, response);
    if (await response.get_code() == AMapTrackErrorCode.AMapTrackErrorOK) {
      if (_onQueryDistance != null) {
        _onQueryDistance(await response.get_distance());
      }
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(
        (await response.get_code()).toValue(),
        await response.get_info(),
      ));
    }
  }

  @override
  Future<void> onQueryTrackHistoryAndDistanceDone_response(
    AMapTrackQueryTrackHistoryAndDistanceRequest request,
    AMapTrackQueryTrackHistoryAndDistanceResponse response,
  ) async {
    super.onQueryTrackHistoryAndDistanceDone_response(request, response);
    if (await response.get_code() == AMapTrackErrorCode.AMapTrackErrorOK) {
      if (_onQueryHistoryTrack != null) {
        _onQueryHistoryTrack(await HistoryTrack.ios(response));
      }
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(
        (await response.get_code()).toValue(),
        await response.get_info(),
      ));
    }
  }

  @override
  Future<void> onQueryTrackInfoDone_response(
    AMapTrackQueryTrackInfoRequest request,
    AMapTrackQueryTrackInfoResponse response,
  ) async {
    super.onQueryTrackInfoDone_response(request, response);
    if (await response.get_code() == AMapTrackErrorCode.AMapTrackErrorOK) {
      if (_onQueryTrack != null) {
        _onQueryTrack(
          [
            for (final item in await response.get_tracks())
              await Track.ios(item)
          ],
        );
      }
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(
        (await response.get_code()).toValue(),
        await response.get_info(),
      ));
    }
  }

  @override
  Future<void> didFailWithError_associatedRequest(
    NSError error,
    dynamic request,
  ) async {
    super.didFailWithError_associatedRequest(error, request);
    if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(await error.code, await error.description));
    }
  }
}

class _AndroidDelegate extends java_lang_Object
    with
        com_amap_api_track_OnTrackLifecycleListener,
        com_amap_api_track_query_model_OnTrackListener {
  ValueChanged<QueryTerminalResponse> _onQueryTerminalResponse;
  ValueChanged<int> _onAddTerminal;
  ValueChanged<int> _onAddTrack;
  VoidCallback _onStartService;
  VoidCallback _onStopService;
  VoidCallback _onStartGather;
  VoidCallback _onStopGather;
  ValueChanged<TrackPoint> _onQueryLastPoint;
  ValueChanged<double> _onQueryDistance;
  ValueChanged<HistoryTrack> _onQueryHistoryTrack;
  ValueChanged<List<Track>> _onQueryTrack;

  ValueChanged<TrackError> _onRequestFailed;

  @override
  Future<void> onStartTrackCallback(int status, String msg) async {
    super.onStartTrackCallback(status, msg);
    if (status == com_amap_api_track_ErrorCode_TrackListen.START_TRACK_SUCEE ||
        status ==
            com_amap_api_track_ErrorCode_TrackListen
                .START_TRACK_SUCEE_NO_NETWORK ||
        status ==
            com_amap_api_track_ErrorCode_TrackListen
                .START_TRACK_ALREADY_STARTED) {
      if (_onStartService != null) {
        _onStartService();
      }
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(status, msg));
    }
  }

  @override
  Future<void> onStopTrackCallback(int status, String msg) async {
    super.onStopTrackCallback(status, msg);
    debugPrint('停止服务');
    if (status == com_amap_api_track_ErrorCode_TrackListen.STOP_TRACK_SUCCE) {
      if (_onStopService != null) {
        _onStopService();
      }
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(status, msg));
    }
  }

  @override
  Future<void> onStartGatherCallback(int status, String msg) async {
    super.onStartGatherCallback(status, msg);
    if (status == com_amap_api_track_ErrorCode_TrackListen.START_GATHER_SUCEE ||
        status ==
            com_amap_api_track_ErrorCode_TrackListen
                .START_GATHER_ALREADY_STARTED) {
      debugPrint('开始收集定位信息');
      if (_onStartGather != null) {
        _onStartGather();
      }
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(status, msg));
    }
  }

  @override
  Future<void> onStopGatherCallback(int status, String msg) async {
    super.onStopGatherCallback(status, msg);
    if (status == com_amap_api_track_ErrorCode_TrackListen.STOP_GATHER_SUCCE ||
        status ==
            com_amap_api_track_ErrorCode_TrackListen
                .STOP_GATHER_GATHER_NOT_STARTED) {
      debugPrint('停止收集定位信息');
      if (_onStopGather != null) {
        _onStopGather();
      }
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(status, msg));
    }
  }

  @override
  Future<void> onAddTrackCallback(
    com_amap_api_track_query_model_AddTrackResponse var1,
  ) async {
    super.onAddTrackCallback(var1);
    if (await var1.isSuccess()) {
      if (_onAddTrack != null) {
        _onAddTrack(await var1.getTrid());
      }
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(
        await var1.getErrorCode(),
        await var1.getErrorMsg(),
      ));
    }
  }

  @override
  Future<void> onQueryTerminalCallback(
    com_amap_api_track_query_model_QueryTerminalResponse var1,
  ) async {
    super.onQueryTerminalCallback(var1);
    if (_onQueryTerminalResponse != null) {
      _onQueryTerminalResponse(await QueryTerminalResponse.android(var1));
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(
        await var1.getErrorCode(),
        await var1.getErrorMsg(),
      ));
    }
  }

  @override
  Future<void> onCreateTerminalCallback(
    com_amap_api_track_query_model_AddTerminalResponse var1,
  ) async {
    super.onCreateTerminalCallback(var1);
    if (await var1.isSuccess()) {
      if (_onAddTerminal != null) {
        _onAddTerminal(await var1.getTid());
      }
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(
        await var1.getErrorCode(),
        await var1.getErrorMsg(),
      ));
    }
  }

  @override
  Future<void> onLatestPointCallback(
    com_amap_api_track_query_model_LatestPointResponse var1,
  ) async {
    super.onLatestPointCallback(var1);
    if (await var1.isSuccess()) {
      if (_onQueryLastPoint != null) {
        _onQueryLastPoint(await TrackPoint.android(
            await var1.getLatestPoint().then((value) => value.getPoint())));
      }
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(
        await var1.getErrorCode(),
        await var1.getErrorMsg(),
      ));
    }
  }

  @override
  Future<void> onDistanceCallback(
    com_amap_api_track_query_model_DistanceResponse var1,
  ) async {
    super.onDistanceCallback(var1);
    if (await var1.isSuccess()) {
      if (_onQueryDistance != null) {
        _onQueryDistance(await var1.getDistance());
      }
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(
        await var1.getErrorCode(),
        await var1.getErrorMsg(),
      ));
    }
  }

  @override
  Future<void> onHistoryTrackCallback(
    com_amap_api_track_query_model_HistoryTrackResponse var1,
  ) async {
    super.onHistoryTrackCallback(var1);
    if (await var1.isSuccess()) {
      if (_onQueryHistoryTrack != null) {
        _onQueryHistoryTrack(
            await HistoryTrack.android(await var1.getHistoryTrack()));
      }
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(
        await var1.getErrorCode(),
        await var1.getErrorMsg(),
      ));
    }
  }

  @override
  Future<void> onQueryTrackCallback(
      com_amap_api_track_query_model_QueryTrackResponse var1) async {
    super.onQueryTrackCallback(var1);
    if (await var1.isSuccess()) {
      if (_onQueryTrack != null) {
        _onQueryTrack(
          [
            for (final item in await var1.getTracks()) await Track.android(item)
          ],
        );
      }
    } else if (_onRequestFailed != null) {
      _onRequestFailed(TrackError(
        await var1.getErrorCode(),
        await var1.getErrorMsg(),
      ));
    }
  }
}
