import 'package:amap_track_fluttify/src/android/android.export.g.dart';
import 'package:amap_track_fluttify/src/ios/ios.export.g.dart';

class QueryTerminalResponse {
  static Future<QueryTerminalResponse> android(
    com_amap_api_track_query_model_QueryTerminalResponse model,
  ) async {
    final result = QueryTerminalResponse._();

    result..terminalId = await model.getTid();

    return result;
  }

  static Future<QueryTerminalResponse> ios(
    AMapTrackQueryTerminalResponse model,
  ) async {
    final result = QueryTerminalResponse._();

    int terminalId = -1;
    final terminalList = await model.get_terminals();
    if (terminalList.isNotEmpty) {
      terminalId = int.parse(await terminalList[0].get_tid());
    }
    result..terminalId = terminalId;

    return result;
  }

  QueryTerminalResponse._();

  /// 获取终端id
  ///
  /// 如果返回-1表示还没有创建该终端
  int terminalId;

  @override
  String toString() {
    return 'QueryTerminalResponse{terminalId: $terminalId}';
  }
}

class TrackPoint {
  static Future<TrackPoint> android(
      com_amap_api_track_query_entity_Point model) async {
    final result = TrackPoint._();
    result
      ..coordinate = LatLng(await model.getLat(), await model.getLng())
      ..time = DateTime.fromMillisecondsSinceEpoch(await model.getTime())
      ..accuracy = await model.getAccuracy()
      ..direction = await model.getDirection()
      ..height = await model.getHeight()
      ..speed = await model.getSpeed();
    return result;
  }

  static Future<TrackPoint> ios(AMapTrackPoint model) async {
    final result = TrackPoint._();
    final coordinate = await model.get_coordinate();
    result
      ..coordinate =
          LatLng(await coordinate.latitude, await coordinate.longitude)
      ..time = DateTime.fromMillisecondsSinceEpoch(await model.get_locateTime())
      ..accuracy = await model.get_accuracy()
      ..direction = await model.get_direction()
      ..height = await model.get_height()
      ..speed = await model.get_speed();
    return result;
  }

  TrackPoint._();

  /// 经纬度
  LatLng coordinate;

  /// 时间
  DateTime time;

  /// 精度
  double accuracy;

  /// 方向
  double direction;

  /// 高度
  double height;

  /// 速度
  double speed;

  @override
  String toString() {
    return 'TrackPoint{coordinate: ${coordinate.latitude}, ${coordinate.longitude}, time: $time, accuracy: $accuracy, direction: $direction, height: $height, speed: $speed}';
  }
}

class HistoryTrack {
  static Future<HistoryTrack> android(
      com_amap_api_track_query_entity_HistoryTrack model) async {
    final result = HistoryTrack._();

    final points = await model.getPoints();
    final latitudeBatch = await points.getLat_batch();
    final longitudeBatch = await points.getLng_batch();
    final coordinateBatch = [
      for (int i = 0; i < points.length; i++)
        LatLng(latitudeBatch[i], longitudeBatch[i])
    ];
    final timeBatch = [
      for (final timestamp in await points.getTime_batch())
        DateTime.fromMillisecondsSinceEpoch(timestamp)
    ];
    final accuracyBatch = await points.getAccuracy_batch();
    final directionBatch = await points.getDirection_batch();
    final heightBatch = await points.getHeight_batch();
    final speedBatch = await points.getSpeed_batch();
    result
      ..trackPointList = [
        for (int i = 0; i < points.length; i++)
          TrackPoint._()
            ..coordinate = coordinateBatch[i]
            ..time = timeBatch[i]
            ..accuracy = accuracyBatch[i]
            ..direction = directionBatch[i]
            ..height = heightBatch[i]
            ..speed = speedBatch[i]
      ]
      ..count = await model.getCount()
      ..distance = await model.getDistance();

    return result;
  }

  static Future<HistoryTrack> ios(
      AMapTrackQueryTrackHistoryAndDistanceResponse model) async {
    final result = HistoryTrack._();

    final points = await model.get_points();

    final coordinateBatch = await points.get_coordinate_batch();
    final latitudeBatch = await coordinateBatch.latitudeBatch;
    final longitudeBatch = await coordinateBatch.longitudeBatch;
    final _coordinateBatch = [
      for (int i = 0; i < points.length; i++)
        LatLng(latitudeBatch[i], longitudeBatch[i])
    ];
    final timeBatch = [
      for (final timestamp in await points.get_locateTime_batch())
        DateTime.fromMillisecondsSinceEpoch(timestamp)
    ];
    final accuracyBatch = await points.get_accuracy_batch();
    final directionBatch = await points.get_direction_batch();
    final heightBatch = await points.get_height_batch();
    final speedBatch = await points.get_speed_batch();
    result
      ..trackPointList = [
        for (int i = 0; i < points.length; i++)
          TrackPoint._()
            ..coordinate = _coordinateBatch[i]
            ..time = timeBatch[i]
            ..accuracy = accuracyBatch[i]
            ..direction = directionBatch[i]
            ..height = heightBatch[i]
            ..speed = speedBatch[i]
      ]
      ..count = await model.get_count()
      ..distance = await model.get_distance();

    return result;
  }

  HistoryTrack._();

  /// 轨迹点
  List<TrackPoint> trackPointList;

  /// 点数量
  int count;

  /// 距离
  num distance;

  @override
  String toString() {
    return 'HistoryTrack{trackPointList: $trackPointList, count: $count, distance: $distance}';
  }
}

/// 单条轨迹
class Track {
  static Future<Track> android(
      com_amap_api_track_query_entity_Track model) async {
    final result = Track._();

    final points = await model.getPoints();
    final latitudeBatch = await points.getLat_batch();
    final longitudeBatch = await points.getLng_batch();
    final coordinateBatch = [
      for (int i = 0; i < points.length; i++)
        LatLng(latitudeBatch[i], longitudeBatch[i])
    ];
    final timeBatch = [
      for (final timestamp in await points.getTime_batch())
        DateTime.fromMillisecondsSinceEpoch(timestamp)
    ];
    final accuracyBatch = await points.getAccuracy_batch();
    final directionBatch = await points.getDirection_batch();
    final heightBatch = await points.getHeight_batch();
    final speedBatch = await points.getSpeed_batch();
    result
      ..trackPointList = [
        for (int i = 0; i < points.length; i++)
          TrackPoint._()
            ..coordinate = coordinateBatch[i]
            ..time = timeBatch[i]
            ..accuracy = accuracyBatch[i]
            ..direction = directionBatch[i]
            ..height = heightBatch[i]
            ..speed = speedBatch[i]
      ]
      ..count = await model.getCount()
      ..distance = await model.getDistance()
      ..lastingTime = Duration(seconds: await model.getTime());

    return result;
  }

  static Future<Track> ios(AMapTrackBasicTrack model) async {
    final result = Track._();

    final points = await model.get_points();

    final coordinateBatch = await points.get_coordinate_batch();
    final latitudeBatch = await coordinateBatch.latitudeBatch;
    final longitudeBatch = await coordinateBatch.longitudeBatch;
    final _coordinateBatch = [
      for (int i = 0; i < points.length; i++)
        LatLng(latitudeBatch[i], longitudeBatch[i])
    ];
    final timeBatch = [
      for (final timestamp in await points.get_locateTime_batch())
        DateTime.fromMillisecondsSinceEpoch(timestamp)
    ];
    final accuracyBatch = await points.get_accuracy_batch();
    final directionBatch = await points.get_direction_batch();
    final heightBatch = await points.get_height_batch();
    final speedBatch = await points.get_speed_batch();
    result
      ..trackPointList = [
        for (int i = 0; i < points.length; i++)
          TrackPoint._()
            ..coordinate = _coordinateBatch[i]
            ..time = timeBatch[i]
            ..accuracy = accuracyBatch[i]
            ..direction = directionBatch[i]
            ..height = heightBatch[i]
            ..speed = speedBatch[i]
      ]
      ..count = await model.get_counts()
      ..distance = await model.get_distance()
      ..lastingTime = Duration(seconds: await model.get_lastingTime());

    return result;
  }

  Track._();

  /// 轨迹点
  List<TrackPoint> trackPointList;

  /// 点数量
  int count;

  /// 距离
  num distance;

  /// 持续时间
  Duration lastingTime;

  @override
  String toString() {
    return 'Track{trackPointList: $trackPointList, count: $count, distance: $distance, lastingTime: $lastingTime}';
  }
}

/// 错误信息
///
/// 官方错误码对照表 https://lbs.amap.com/api/android-track/guide/tools/errorcode
class TrackError {
  TrackError(this.code, this.desc);

  /// 错误码
  final int code;

  /// 错误描述
  final String desc;

  @override
  String toString() {
    return 'TrackError{code: $code, desc: $desc}';
  }
}
